package com.yihu.ehr.redis.cache.service;


import com.yihu.ehr.redis.schema.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;


/**
 * Redis管理
 *
 * @author hzp add at 20170425
 */
@Service
public class RedisService {

    @Autowired
    private HealthProblemDictKeySchema healthProblemDictKeySchema;
    @Autowired
    private Icd10KeySchema icd10KeySchema;
    @Autowired
    private OrgKeySchema orgKeySchema;
    @Autowired
    private RsAdapterMetaKeySchema rsAdapterMetaKeySchema;
    @Autowired
    private RsMetadataKeySchema rsMetadataKeySchema;
    @Autowired
    private StdCdaVersionKeySchema stdVersionKeySchema;
    @Autowired
    private StdDataSetKeySchema stdDataSetKeySchema;
    @Autowired
    private StdMetaDataKeySchema stdMetaDataKeySchema;

   /* @Autowired
    private IndicatorsDictKeySchema indicatorsDictKeySchema;*/

    /**
     * 获取健康问题redis
     *
     * @return
     */
    public String getHealthProblem(String key) {
        return healthProblemDictKeySchema.get(key);
    }

    /**
     * 获取ICD10名称 redis
     */
    public String getIcd10Name(String key) {
        return icd10KeySchema.get(key);
    }

    /**
     * 获取ICD10对应健康问题 redis
     */
    public String getHpCodeByIcd10(String key) {
        return icd10KeySchema.getHpCode(key);
    }

    /**
     * 获取ICD10慢病信息 redis
     */
    public String getChronicInfo(String key) {
        return icd10KeySchema.getChronicInfo(key);
    }

    /**
     * 获取机构名称redis
     *
     * @return
     */
    public String getOrgName(String key) {
        return orgKeySchema.get(key);
    }

    /**
     * 获取机构区域redis
     *
     * @return
     */
    public String getOrgArea(String key) {
        return orgKeySchema.getOrgArea(key);
    }

    /**
     * 获取机构Saas区域权限范围redis
     *
     * @return
     */
    public String getOrgSaasArea(String key) {
        return orgKeySchema.getOrgSaasArea(key);
    }

    /**
     * 获取机构Saas机构权限范围redis
     *
     * @return
     */
    public String getOrgSaasOrg(String key) {
        String saasOrg = orgKeySchema.getOrgSaasOrg(key);
        //未设置权限，默认自身机构
        if (StringUtils.isEmpty(saasOrg)) {
            saasOrg = key;
        }
        return saasOrg;
    }

    //------------------------------------ 资源化相关 START -------------------------------------------------------

    /**
     * 获取资源化数据元映射 redis
     *
     * @return
     */
    public String getRsAdapterMetaData(String cdaVersion, String srcDataSetCode, String srcMetadataCode) {
        return rsAdapterMetaKeySchema.getMetaData(cdaVersion, srcDataSetCode, srcMetadataCode);
    }

    /**
     * 获取资源化数据元映射 redis
     *
     * @return
     */
    public String getRsMetadataDict(String key) {
        return rsMetadataKeySchema.get(key);
    }

    //------------------------------------ 资源化相关 END -------------------------------------------------------

    //------------------------------------ 标准相关 START -------------------------------------------------------

    /**
     * 获取标准版本 redis
     */
    public String getStdVersion(String key) {
        return stdVersionKeySchema.get(key);
    }

    /**
     * 获取标准数据集代码 redis
     */
    public String getDataSetCode(String version, String id) {
        return stdDataSetKeySchema.dataSetCode(version, id);
    }

    /**
     * 获取标准数据集名称 redis
     */
    public String getDataSetName(String version, String id) {
        return stdDataSetKeySchema.dataSetName(version, id);
    }

    /**
     * 获取标准数据集名称 redis
     */
    public String getDataSetNameByCode(String version, String code) {
        return stdDataSetKeySchema.dataSetNameByCode(version, code);
    }

    /**
     * 获取标准数据集--主从表 redis
     */
    public Boolean getDataSetMultiRecord(String version, String code) {
        return stdDataSetKeySchema.dataSetMultiRecord(version, code);
    }

    /**
     * 获取标准数据元对应类型 redis
     */
    public String getMetaDataType(String version, String dataSetCode, String innerCode) {

        return stdMetaDataKeySchema.metaDataType(version, dataSetCode, innerCode);
    }

    /**
     * 获取数据元格式
     *
     * @param version
     * @param dataSetCode
     * @param innerCode
     * @return
     */
    public String getMetaDataFormat(String version, String dataSetCode, String innerCode) {

        return stdMetaDataKeySchema.metaDataFormat(version, dataSetCode, innerCode);
    }

    /**
     * 获取数据元是否为空
     *
     * @param version
     * @param dataSetCode
     * @param innerCode
     * @return
     */
    public Boolean isMetaDataNullable(String version, String dataSetCode, String innerCode) {

        return stdMetaDataKeySchema.metaDataNullable(version, dataSetCode, innerCode);
    }

    /**
     * 获取标准数据元对应字典 redis
     */
    public String getMetaDataDict(String version, String dataSetCode, String innerCode) {
        return stdMetaDataKeySchema.metaDataDict(version, dataSetCode, innerCode);
    }

    /**
     * 获取标准数据字典对应值 redis
     */
    public String getDictEntryValue(String version, String dictId, String entryCode) {

        return stdMetaDataKeySchema.dictEntryValue(version, dictId, entryCode);
    }

    /**
     * 判断值域编码是否存在
     *
     * @param version
     * @param dictId
     * @param entryCode
     * @return
     */
    public Boolean isDictCodeExist(String version, String dictId, String entryCode) {
        return stdMetaDataKeySchema.isDictCodeExist(version, dictId, entryCode);
    }

    /**
     * 判断值域编码是否存在
     *
     * @param version
     * @param dictId
     * @param entryValue
     * @return
     */
    public Boolean isDictValueExist(String version, String dictId, String entryValue) {
        return stdMetaDataKeySchema.isDictValueExist(version, dictId, entryValue);
    }
    //------------------------------------ 标准相关 END -------------------------------------------------------

    /**
     * 获取指标 redis
     *
     * @return
     */
    /*public String getIndicators(String key) {
        return indicatorsDictKeySchema.get(key);
    }*/
}
